import requests
import json
import warnings
from pprint import pprint

warnings.simplefilter( "ignore" )

def pairing_stage_1( ip_address ):
	headers = {
		"Content-Type": "application/json" ,
	}
	data = {
		"_url": "/pairing/start" ,
		"DEVICE_ID": "pyvizio" ,
		"DEVICE_NAME": "Python Vizio" ,
	}
	url = f"https://{ip_address}:7345/pairing/start"
	response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
	response.raise_for_status()

	# Should Return
	# {
	# 	'ITEM': {'CHALLENGE_TYPE': 1, 'PAIRING_REQ_TOKEN': 802927 } ,
	# 	'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}
	# }
	result = json.loads( response.text )
	pprint( result )
	return result

def pairing_stage_2( ip_address , pairing_request_token , code_displayed_on_tv ):
	headers = {
		"Content-Type": "application/json" ,
	}
	data = {
		"_url": "/pairing/pair" ,
		"DEVICE_ID": "pyvizio" ,
		"DEVICE_NAME": "Python Vizio" ,
		"CHALLENGE_TYPE": 1 ,
		"PAIRING_REQ_TOKEN": pairing_request_token ,
		"RESPONSE_VALUE": str( code_displayed_on_tv )
	}
	print( data )
	url = f"https://{ip_address}:7345/pairing/pair"
	response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
	response.raise_for_status()

	# Should Return
	# {
	# 	'ITEM': {'AUTH_TOKEN': 'Zhehzvszfq' } ,
	# 	'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}
	# }
	result = json.loads( response.text )
	pprint( result )
	return result


def get_volume( ip_address , access_token ):
	headers = {
		'AUTH': access_token
	}
	url = f"https://{ip_address}:7345/menu_native/dynamic/tv_settings/audio/volume"
	response = requests.get( url , headers=headers , verify=False )
	response.raise_for_status()

	# Should Return
	# {'HASHLIST': [2308455925, 729988045],
	#  'ITEMS': [{'CNAME': 'volume',
	#             'ENABLED': 'FALSE',
	#             'HASHVAL': 1731828541,
	#             'NAME': 'Volume',
	#             'TYPE': 'T_VALUE_V1',
	#             'VALUE': 10}],
	#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
	#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
	#  'URI': '/menu_native/dynamic/tv_settings/audio/volume'}

	result = json.loads( response.text )
	pprint( result )
	return result


if __name__ == "__main__":

	#pairing_stage_1( "192.168.1.102" )

	#pairing_stage_2( "192.168.1.102" , 434136 , 3232 )

	get_volume( "192.168.1.102" , "Zhehzvszfq" )

	# Found where it was calling with: https://codeburst.io/how-i-use-python-debugger-to-fix-code-279f11f75866?gi=fed84a534cf9
	# Put debugger on cli.py --> pair()

	# Somehow it calls the sitepackages , I just put print commands in
	# /home/morphs/.local/lib/python3.8/site-packages/pyvizio/_protocol.py

	# Continue Testing via
	# cd /home/morphs/WORKSPACE/PYTHON/VizioController/pyvizio/
	# python3 pyvizio/cli.py --ip=192.168.1.102:7345 --auth=Zhehzvszfq get-volume-level

	# Just keep converting http request into functions, then to class


#!/usr/bin/env python3

# Based on https://github.com/vkorn/pyvizio

import discover
import api
from pprint import pprint

class VizioController:

	def __init__( self , options={} ):
		self.options = options
		if "mac_address" not in options:
			print("you have to send the mac adress of the tv")
			sys.exit( 1 )
		if "ip" not in options:
			self.discover = discover.Discover( options )
			self.ip = self.discover.find_tv()
			options["ip"] = self.ip
		else:
			self.ip = options["ip"]
		if "request_token" in options:
			if "code_displayed_on_tv" in options:
				self.api = api.API(options)
				options["access_token"] = self.api.pairing_stage_2( self.ip , options["request_token"] , options["code_displayed_on_tv"] )
		if "access_token" not in options:
			self.api = api.API(options)
			request_token = self.api.pairing_stage_1( self.ip )
			print( f"Ok , now rerun this and set options['request_token']: {request_token}" )
			print( f"and options['code_displayed_on_tv']: code on tv" )
			sys.exit( 1 )

		self.api = api.API( options )
		print( "Retrieving Current Settings" )
		self.current_volume = self.api.get_volume()
		self.audio_settings = self.api.get_audio_settings()
		self.audio_settings_options = self.api.get_all_audio_settings_options()
		self.current_input = self.api.get_current_input()
		self.available_inputs = self.api.get_available_inputs()
		self.current_app = self.api.get_current_app()
		self.settings_types = self.api.get_settings_types()
		self.settings = {}
		for index , setting in enumerate( self.settings_types["ITEMS"] ):
			options = self.api.get_all_settings_for_type( setting["CNAME"] )
			options = [ x["CNAME"] for x in options["ITEMS"] ]
			self.settings[setting["CNAME"]] = {}
			for option_index , option in enumerate( options ):
				self.settings[setting["CNAME"]][ option ] = self.api.get_setting( setting["CNAME"] , option )
		pprint( self.settings )


if __name__ == "__main__":
	tv = VizioController({
			"name": "Loft TV" ,
			"mac_address": "2c:64:1f:25:6b:3c" ,
			"ip": "192.168.1.100" ,
			"access_token": "Zhehzvszfq"
		})
	#print( tv.ip )

	#current_volume = tv.api.get_volume()
	#tv.api.volume_up()
	#tv.api.volume_down()
	#audio_settings = tv.api.get_audio_settings()
	#tv_speakers = tv.api.get_audio_setting( "tv_speakers" )
	#all_audio_settings_options = tv.api.get_all_audio_settings_options()
	#tv_speakers = tv.api.get_audio_settings_options( "tv_speakers" )
	#tv.api.set_audio_setting( "mute" , "Off" )

	#current_input = tv.api.get_current_input()
	#available_inputs = tv.api.get_available_inputs()
	# tv.api.set_input( "HDMI-1" )
	# tv.api.set_input( "HDMI-2" )
	#tv.api.cycle_input()

	#settings_types = tv.api.get_settings_types()
	#audio_settings = tv.api.get_all_settings_for_type( "audio" )
	#backlight = tv.api.get_setting( "picture" , "backlight" )
	#picture_settings = tv.api.get_all_settings_options_for_type( "picture" )
	#backlight_setting = tv.api.get_settings_option( "picture" , "backlight" )
	#tv.api.set_settings_option( "picture" , "backlight" , 100 )

	#tv.api.launch_app_config( "1" , 3 )
	#current_app = tv.api.get_current_app()

	# Whenever this is done, post it here:
	# https://github.com/vkorn/pyvizio/issues/5


import setuptools

setuptools.setup(
	name="viziocontroller",
	version="0.0.1",
	author="7435171",
	author_email="48723247842@protonmail.com",
	description="Vizio Controller",
	url="https://github.com/48723247842/VizioController",
	packages=setuptools.find_packages(),
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	python_requires='>=3.6',
)

install_requires = [
	'sys' ,
	'os' ,
	'subprocess' ,
	'socket' ,
	'netifaces' ,
	'platform' ,
	'requests' ,
	'json' ,
	'warnings' ,
	'pprint' ,
]
